#
# FtpCube
# Copyright (C) 2001 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

defaults = {
	'host'          : None,
	'port'          : 21,
	'username'      : 'anonymous',
	'password'      : 'guest@anon.com',
	'timeout'       : 120,
	'localdir'      : None,
	'limit'         : None,
	'thread_idle'   : 30,
	'main_idle'     : 300,
	'delay'         : 5,
	'retries'       : 5,
	'proxy'         : 0,
	'remotedir'     : None,
	'logging'       : 0,
	'cmd_log'       : 'commands.txt',
	'download_log'  : 'download.txt',
	'upload_log'    : 'upload.txt',
	'local_color'   : (0.21482819734186889, 0.09512286143262505, 0.43703703703703706),
	'remote_color'  : (0.16822981105023022, 0.37037037037037035, 0.18973221780044075),
	'error_color'   : (0.45185185185185184, 0.092204513827356852, 0.15133532313537504),
	'transfer_type' : 0,
	'win_size'      : (0, 0)
}
